VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CApiPrinterDriver"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_HelpID = 2672
Attribute VB_Description = "This is the printer driver:EventVB~ApiPrinterDriver object that relates to the driver software installed for this printer."
Option Explicit

' This is our CApiPrinterDriver object that relates to the driver software installed.
' A printer driver is typically provided by the printer manufacturer.
' Different manufacturers may provide significantly different functionality in their
' respective drivers.
' This class wraps the most common (and useful) subset of this functionality and
' should be pretty safe in daily use (heavily tested with Grafikhusets printers).

Private Const CLASS_NAME = "CApiPrinterDriver"

' The version of the Windows operating system for which
' a driver was specifically written.
Public Enum PrinterDriverOSVersion
     ' Windows 95, 98 or Me
     DRIVER_OS_WIN9x = &H0
     ' Windows NT 3.51
     DRIVER_OS_NT351 = &H1
     ' Windows NT4
     DRIVER_OS_NT4 = &H2
     ' Windows 2000 (and XP)
     DRIVER_OS_WIN2K = &H3
End Enum

Private Type DRIVER_INFO_1
    lpDriverName As String
End Type

Private Type DRIVER_INFO_2
     lVersion As PrinterDriverOSVersion
     lpDriverName As String
     lpEnvironment As String
     lpDriverPath As String
     lpDataFile As String
     lpConfigFile As String
End Type

Private Type DRIVER_INFO_3
     lVersion As PrinterDriverOSVersion
     lpDriverName As String
     lpEnvironment As String
     lpDriverPath As String
     lpDataFile As String
     lpConfigFile As String
     lpHelpFile As String
     lpDependentFiles As String
     lpMonitorName As String
     lpDefaultDataType As String
End Type
'
' The following types are only supported by drivers
' written for Win2K and WinXP
Private Type DRIVER_INFO_4
     lVersion As PrinterDriverOSVersion
     lpDriverName As String
     lpEnvironment As String
     lpDriverPath As String
     lpDataFile As String
     lpConfigFile As String
     lpHelpFile As String
     lpDependentFiles As String
     lpMonitorName As String
     lpDefaultDataType As String
     lpCompatiblePreviousNames As String
End Type
'
' API Functions used...
Private Declare Function GetPrinterDriver Lib "winspool.drv" Alias "GetPrinterDriverA" _
                  (ByVal hPrinter As Long, _
                   ByVal lpEnvironment As String, _
                   ByVal Level As Long, _
                   pDriverBuffer As Long, _
                   ByVal cBufferSize As Long, _
                   cBufferSizeRequired As Long) As Long
                   
Private Declare Function EnumPrinterDrivers Lib "winspool.drv" Alias "EnumPrinterDriversA" _
                  (ByVal lpServerName As String, _
                   ByVal lpEnvironment As String, _
                   ByVal Level As Long, _
                   lpDriverBuffer As Long, _
                   ByVal cBufferSize As Long, _
                   cBufferSizeRequired As Long) As Long
                   
Private Type PRINTER_DEFAULTS
    pDatatype As String
    pDevMode As Long
    DesiredAccess As Long
End Type

Private Declare Function OpenPrinter Lib "winspool.drv" _
        Alias "OpenPrinterA" (ByVal pPrinterName As String, _
        phPrinter As Long, pDefault As PRINTER_DEFAULTS) As Long

Private Declare Function ClosePrinter Lib "winspool.drv" _
        (ByVal hPrinter As Long) As Long

' Private member variables....
Private mhPrinter As Long
Private mEnvironment As String

Private mDRIVER_INFO_1 As DRIVER_INFO_1
Private mDRIVER_INFO_2 As DRIVER_INFO_2
Private mDRIVER_INFO_3 As DRIVER_INFO_3
Private mDRIVER_INFO_4 As DRIVER_INFO_4

Private m_PPDInformation As CPPDInformation

Private bChanged As Boolean
Friend Property Get Changed() As Boolean
    Changed = bChanged
End Property
Friend Property Let Changed(ByVal newChanged As Boolean)
    bChanged = newChanged
End Property

Friend Property Get PPDInformation(Optional ByVal bRefresh As Boolean = False) As CPPDInformation
    If IsPostScript Then
        If bRefresh Then
            Set PPDInformation = GetPPDInformation
        Else
            Set PPDInformation = m_PPDInformation
        End If
    Else
        Set PPDInformation = Nothing
    End If
End Property

Public Property Get CompatiblePreviousDriverNames() As String
Attribute CompatiblePreviousDriverNames.VB_HelpID = 2673

    Call RefreshDriverInfo(4)
    CompatiblePreviousDriverNames = mDRIVER_INFO_4.lpCompatiblePreviousNames
    
End Property

Public Property Get ConfigurationFile() As String
Attribute ConfigurationFile.VB_HelpID = 2674

    Call RefreshDriverInfo(2)
    ConfigurationFile = mDRIVER_INFO_2.lpConfigFile
    
End Property

Public Property Get DataFile() As String
Attribute DataFile.VB_HelpID = 2675

    Call RefreshDriverInfo(2)
    DataFile = mDRIVER_INFO_2.lpDataFile
    
End Property

Public Property Get DefaultDataType() As String
Attribute DefaultDataType.VB_HelpID = 2676

    Call RefreshDriverInfo(3)
    DefaultDataType = mDRIVER_INFO_3.lpDefaultDataType
    
End Property

Public Property Get Environment() As String
Attribute Environment.VB_HelpID = 2677
    
    Call RefreshDriverInfo(2)
    Environment = mDRIVER_INFO_2.lpEnvironment
    
End Property

Friend Property Let PrinterName(ByVal bGetPPDInfoNow As Boolean, ByVal newName As String)
    
    Dim pDefaults As PRINTER_DEFAULTS
    Dim lret As Long
    
    If mhPrinter <> 0 Then
        lret = ClosePrinter(mhPrinter)
        mhPrinter = 0
    End If
    
    lret = OpenPrinter(newName, mhPrinter, pDefaults)
    
    If bGetPPDInfoNow Then Call GetPPDInformation(True)
        
End Property

Public Property Get HelpFile() As String
Attribute HelpFile.VB_HelpID = 2678

    Call RefreshDriverInfo(3)
    HelpFile = mDRIVER_INFO_3.lpHelpFile
    
End Property

Public Property Get MonitorName() As String
Attribute MonitorName.VB_HelpID = 2679

    Call RefreshDriverInfo(3)
    MonitorName = mDRIVER_INFO_3.lpMonitorName
    
End Property


Public Property Get Name() As String
Attribute Name.VB_HelpID = 2680
    
    Call RefreshDriverInfo(2)
    Name = mDRIVER_INFO_2.lpDriverName

End Property

Public Property Get OperatingSystemVersion() As PrinterDriverOSVersion
Attribute OperatingSystemVersion.VB_HelpID = 2681
    
    Call RefreshDriverInfo(2)
    OperatingSystemVersion = mDRIVER_INFO_2.lVersion

End Property

Public Property Get Path() As String
Attribute Path.VB_HelpID = 2682

    Call RefreshDriverInfo(2)
    Path = mDRIVER_INFO_2.lpDriverPath
    
End Property

Public Property Get IsPostScript() As Boolean

    If InStr(LCase(DataFile), ".ppd") Then IsPostScript = True

End Property

Private Sub RefreshDriverInfo(ByVal index As Integer)

    Dim lret As Long
    Dim SizeNeeded As Long
    
    Dim buffer() As Long
    '
    ' Find out how much buffer space we're going to need
    ReDim Preserve buffer(0 To 1) As Long
    lret = GetPrinterDriver(mhPrinter, mEnvironment, index, buffer(0), UBound(buffer), SizeNeeded)
    '
    ' Resize the buffer to accomodate that amount
    ReDim Preserve buffer(0 To (SizeNeeded / 4) + 3) As Long
    lret = GetPrinterDriver(mhPrinter, mEnvironment, index, buffer(0), UBound(buffer) * 4, SizeNeeded)
    
    If index < 1 Or index > 4 Then
        '
        ' Index is out of bounds - notify developer of error
        Debug.Print "Error in call to CApiPrinterDriver::RefreshPrinterDriverInfo - invalid index"
    Else
        Select Case index
        Case 1
            With mDRIVER_INFO_1
                .lpDriverName = StringFromPointer(buffer(0), 1024)
            End With
        Case 2
            With mDRIVER_INFO_2
                .lVersion = buffer(0)
                .lpDriverName = StringFromPointer(buffer(1), 1024)
                .lpEnvironment = StringFromPointer(buffer(2), 1024)
                .lpDriverPath = StringFromPointer(buffer(3), 1024)
                .lpDataFile = StringFromPointer(buffer(4), 1024)
                .lpConfigFile = StringFromPointer(buffer(5), 1024)
            End With
        Case 3
            With mDRIVER_INFO_3
                .lVersion = buffer(0)
                .lpDriverName = StringFromPointer(buffer(1), 1024)
                .lpEnvironment = StringFromPointer(buffer(2), 1024)
                .lpDriverPath = StringFromPointer(buffer(3), 1024)
                .lpDataFile = StringFromPointer(buffer(4), 1024)
                .lpConfigFile = StringFromPointer(buffer(5), 1024)
                .lpHelpFile = StringFromPointer(buffer(6), 1024)
                .lpDependentFiles = StringFromPointer(buffer(7), 1024)
                .lpMonitorName = StringFromPointer(buffer(8), 1024)
                .lpDefaultDataType = StringFromPointer(buffer(9), 1024)
            End With
        Case 4
            With mDRIVER_INFO_4
                .lVersion = buffer(0)
                .lpDriverName = StringFromPointer(buffer(1), 1024)
                .lpEnvironment = StringFromPointer(buffer(2), 1024)
                .lpDriverPath = StringFromPointer(buffer(3), 1024)
                .lpDataFile = StringFromPointer(buffer(4), 1024)
                .lpConfigFile = StringFromPointer(buffer(5), 1024)
                .lpHelpFile = StringFromPointer(buffer(6), 1024)
                .lpDependentFiles = StringFromPointer(buffer(7), 1024)
                .lpMonitorName = StringFromPointer(buffer(8), 1024)
                .lpDefaultDataType = StringFromPointer(buffer(9), 1024)
                .lpCompatiblePreviousNames = StringFromPointer(buffer(10), 1024)
            End With
        End Select
    End If

End Sub

Private Function GetPPDInformation(Optional bRefresh As Boolean = True) As CPPDInformation

    Dim fno&
    Dim vBuf$
    Dim Idx&
    
    Dim lpos1&, lpos2&, lpos3&, lpos4&
    
    Const OPEN_UI = "*OpenUI"
    Const CLOSE_UI = "*CloseUI"
    Const KEYWORD_COMMENT = "*%"
    Const ORDER_DEPENDENCY = "*OrderDependency"
    Const DEFAULT_FOR_GROUP = "*Default"
    
    Dim vGroupName$, vDefaultChoice$, vPSStream$, vPSQueryStream$
    
    Dim vPPDFeature As CPPDUIFeature
    Dim vPPDChoice As CPPDUIChoice
    
    Dim bUIGroupIsOpened As Boolean
    Dim bUIGroupOrder As Boolean
    Dim bUIGroupDefault As Boolean
    Dim bQueryStream As Boolean
    
    fno = FreeFile
        
    Screen.MousePointer = vbHourglass
        
    Open DataFile For Input Access Read As #fno
    
    Do While Not EOF(fno)
        Line Input #fno, vBuf
        '
        ' Before we do anything we ensure that this line has some content
        If Len(vBuf) > 2 Then
            '
            ' Filter off PPD comments
            lpos1 = InStr(vBuf, KEYWORD_COMMENT)
            If lpos1 > Len(KEYWORD_COMMENT) Then
                vBuf = Left(vBuf, lpos1 - Len(KEYWORD_COMMENT))
            End If
        
            If Not Left$(vBuf, 2) = "*%" Then
                '
                ' Potentially this line contain useful information. If it contain
                ' the identifier OPEN_UI we have a new PPD Feature with a yet unknown
                ' number of choices a user can select.
                lpos1 = InStr(vBuf, OPEN_UI)
                lpos2 = InStr(Len(OPEN_UI), vBuf, "*") + 1
                lpos3 = InStr(vBuf, ":") + 1
                
                If (lpos1 > 0) And (lpos2 > 0) And (lpos3 > 1) And Not bUIGroupIsOpened Then
                    '
                    ' Hence, we open a new UIGroup and store which type of
                    ' selections we need to present in our User Interface.
                    bUIGroupIsOpened = True
                    Set vPPDFeature = New CPPDUIFeature
                    Call vPPDFeature.InitGroup(Mid(vBuf, lpos2, lpos3 - lpos2 - 1))
                    vGroupName = vPPDFeature.GroupName
                    vPPDFeature.GroupType = Trim(Mid(vBuf, lpos3))
                Else
                    '
                    ' When the UI Group is opened we capture it's contents
                    If bUIGroupIsOpened Then
                        '
                        ' Potentially this line contain a CloseUI command. If so
                        ' we close the key, stop looking for entries and store
                        ' the information.
                        lpos1 = InStr(vBuf, CLOSE_UI)
                        If (lpos1 > 0) Then
                            '
                            ' Store the PPD feature if we have a 'current' that is
                            ' initialized already
                            If Not vPPDChoice Is Nothing Then
                                '
                                ' Save the choice into the FeatureGroup
                              '  vPPDFeature.Choices.Add vPPDChoice
                            End If
                            Set vPPDFeature.DefaultChoice = vPPDFeature.Choices(vDefaultChoice)
                            m_PPDInformation.Add vPPDFeature
                            '
                            ' Mark the group as as closed
                            bUIGroupIsOpened = False
                        '    bQueryStream = False
                        Else
                            '
                            ' If the key is open we save the information it contains.
                            ' First the order dependency (which is recommended, but optional in a PPD)
                            lpos1 = InStr(vBuf, ORDER_DEPENDENCY)
                            lpos2 = InStr(vBuf, ":") + 1
                            If (lpos1 > 0) And (lpos2 > 1) Then
                                Call vPPDFeature.InitOrderDependency(Trim(Mid(vBuf, lpos2)))
                                bUIGroupOrder = True
                            Else
                                bUIGroupOrder = False
                            End If
                            '
                            ' Then the default for this group
                            lpos1 = InStr(vBuf, DEFAULT_FOR_GROUP & vGroupName)
                            lpos2 = InStr(vBuf, ":") + 1
                            If (lpos1 > 0) And (lpos2 > 1) Then
                                vDefaultChoice = Trim(Mid(vBuf, lpos2))
                                bUIGroupDefault = True
                            Else
                                bUIGroupDefault = False
                            End If
                            '
                            ' And finally each choice allowed in this group
                            If Not bUIGroupOrder And Not bUIGroupDefault Then
                                '
                                ' OK, this contain relevant information.
                                '
                                ' First we test for the existance of a translation name (FriendlyName).
                                ' If present it's separated from the GroupName and the colon (:) by a slash.
                                lpos1 = InStr(vBuf, "*" & vGroupName)
                                lpos2 = InStr(vBuf, " ") + 1
                                lpos3 = InStr(vBuf, "/") + 1
                                lpos4 = InStr(vBuf, ":") + 1
                                '
                                ' Ensure that we have a vGroupName in the string. If so this
                                ' is a new UIChoice.
                                If (lpos1 > 0) Then
                                    '
                                    ' Did we already have an initialized UIChoice?
                                    Set vPPDChoice = New CPPDUIChoice
                                    '
                                    ' At this point we start interpreting a new UIChoice.
                                    ' Watch out: The slash may be present in the PS Code.
                                    If (lpos3 > 1) And (lpos3 < lpos4) Then
                                        '
                                        ' OK We have a translation name
                                        vPPDChoice.ChoiceName = Trim(Mid(vBuf, lpos2, lpos3 - lpos2 - 1))
                                        vPPDChoice.TranslationName = Trim(Mid(vBuf, lpos3, lpos4 - lpos3 - 1))
                                        vPSStream = Trim(Mid(vBuf, lpos4))
                                    Else
                                        '
                                        ' No TranslationName found. Hence, use the ChoiceName as indetifier.
                                        vPPDChoice.ChoiceName = Trim(Mid(vBuf, lpos2, lpos4 - lpos2 - 1))
                                        vPPDChoice.TranslationName = vPPDChoice.ChoiceName
                                        vPSStream = Trim(Mid(vBuf, lpos4))
                                    End If
                                    vPPDChoice.CommandString = vPSStream
                                    '
                                    ' Save the choice into the FeatureGroup
                                    vPPDFeature.Choices.Add vPPDChoice
                                '
                                ' Store any query stream for this PPD feature
                                ElseIf InStr(vBuf, "*?" & vGroupName) > 0 Then
                                    bQueryStream = True
                                    vPSQueryStream = ""
                                    vPPDFeature.QueryStream = vPSQueryStream
                                Else
                                    '
                                    ' OK, this is PS code. We add it to the vPSStream variable
                                    ' until '*End' is reached.
                                    lpos1 = InStr(vBuf, "*End")
                                    If (lpos1 = 0) And Not bQueryStream Then
                                        If Left(vPSStream, 1) = """" Then
                                            vPSStream = vPSStream & vBuf
                                        Else
                                            vPSStream = vPSStream & vbCrLf & vBuf
                                        End If
                                        vPPDChoice.CommandString = vPSStream
                                    Else
                                        vPSQueryStream = vPSQueryStream & vBuf
                                        vPPDFeature.QueryStream = vPSQueryStream
                                        bQueryStream = False
                                    End If
                                End If
                                
                            ' Group order or default
                            End If
                            
                        End If
                    
                    End If
                    
                ' bUIGroupIsOpened = False
                End If
                                
            Else
                '
                ' Line was a PPD comment
            End If
        Else
            '
            ' Line was empty
        End If
    
    Loop
    
    Set vPPDFeature = Nothing
    Set vPPDChoice = Nothing
    
    Close #fno

    Set GetPPDInformation = m_PPDInformation

    Screen.MousePointer = vbDefault

End Function

Private Sub Class_Initialize()

    Set m_PPDInformation = New CPPDInformation
    
End Sub

Private Sub Class_Terminate()

    Dim lret As Long

    If mhPrinter <> 0 Then
        lret = ClosePrinter(mhPrinter)
        mhPrinter = 0
    End If

    Set m_PPDInformation = Nothing

End Sub
